@@ -5,7 +5,6 @@
 /*    we now have dummy functions so do not need to   */
 /*    create the file                                 */
 
-#define PDL_CORE      /* For certain ifdefs */
 #include "pdl.h"      /* Data structure declarations */
 #include "pdlcore.h"  /* Core declarations */
 
@@ -117,6 +116,7 @@ pdl* pdl_create(int type) {
      if(type == PDL_TMP) {croak("PDL internal error. FIX!\n");}
 
      it = (pdl*) malloc(sizeof(pdl));
+     memset(it, 0, sizeof(pdl));
      if (it==NULL)
         croak("Out of Memory\n");
 
