@@ -7,8 +7,8 @@ PDL::IO::FITS -- Simple FITS support for PDL
  use PDL;
  use PDL::IO::FITS;
 
- $a = rfits('foo.fits');          # read a FITS file
- $a->wfits('bar.fits');           # write a FITS file
+ $x = rfits('foo.fits');          # read a FITS file
+ $x->wfits('bar.fits');           # write a FITS file
 
 =head1 DESCRIPTION
 
@@ -32,7 +32,7 @@ hashes or (if the module is present) Astro::FITS::Header objects that
 are tied to perl hashes.  Astro::FITS::Header objects provide
 convenient access through the tied hash interface, but also allow you
 to control the card structure in more detail using a separate method
-interface; see the L<Astro::FITS::Header|Astro::FITS::Header>
+interface; see the L<Astro::FITS::Header>
 documentation for details.
 
 =head1 AUTHOR
@@ -79,9 +79,9 @@ BEGIN {
   eval "use Astro::FITS::Header;";
   $PDL::Astro_FITS_Header = (defined $Astro::FITS::Header::VERSION);
   if($PDL::Astro_FITS_Header) {
-    my($a) = $Astro::FITS::Header::VERSION;
-    $a =~ s/[^0-9\.].*//;
-    $PDL::Astro_FITS_Header = 0 if($a < 1.12);
+    my($x) = $Astro::FITS::Header::VERSION;
+    $x =~ s/[^0-9\.].*//;
+    $PDL::Astro_FITS_Header = 0 if($x < 1.12);
   }
 
   unless($PDL::Astro_FITS_Header) {
@@ -117,14 +117,22 @@ Suffix magic:
   $pdl = rfits('file.fits.gz[3]'); # Read 3rd extension
   @pdls = rfits('file.fits');      # Read primary data and extensions
 
+Tilde expansion:
+
+  #expand leading ~ to home directory (using glob())
+  $pdl = rfits '~/filename.fits';
+
   $hdr = rfits('file.fits',{data=>0});  # Options hash changes behavior
 
 In list context, C<rfits> reads the primary image and all possible
 extensions, returning them in the same order that they occurred in the
-file.  In scalar context, the default is to read the primary HDU. One
-can read other HDU's by using the [n] syntax, the second one is [1].
-Currently recognized extensions are IMAGE and BINTABLE.  (See the
-addendum on EXTENSIONS for details).
+file -- except that, by default, the primary HDU is skipped if it
+contains no data.  In scalar context, the default is to read the first
+HDU that contains data. One can read other HDU's by using the [n]
+syntax.  Using the [0] syntax forces a read of the first HDU,
+regardless of whether it contains data or no.  Currently recognized
+extensions are IMAGE and BINTABLE.  (See the addendum on EXTENSIONS
+for details).
 
 C<rfits> accepts several options that may be passed in as a hash ref
 if desired:
@@ -162,6 +170,17 @@ time, unpacking such images transparently and returning the data and header
 as if they were part of a normal IMAGE extension.  Setting "expand" to 0
 delivers the binary table, rather than unpacking it into an image.
 
+=item afh (default=1)
+
+By default rfits uses Astro::FITS::Header tied-hash objects to contain
+the FITS header information.  This permits explicit control over FITS
+card information, and conforms well with the FITS specification.  But
+Astro::FITS::Header objects are about 40-60x more memory intensive
+than comparable perl hashes, and also use ~10x more CPU to manage.
+For jobs where header processing performance is important (e.g. reading 
+just the headers of 1,000 FITS files), set afh to 0 to use the legacy parser
+and get a large boost in speed.
+
 =back
 
 FITS image headers are stored in the output PDL and can be retrieved
@@ -224,7 +243,7 @@ the repeat count) or variable length arrays.)
 Thus, if your table contains a column named C<FOO> with type C<5D>,
 the expression
 
-  $a->{FOO}->((2))
+  $x->{FOO}->((2))
 
 returns a 5-element double-precision PDL containing the values of FOO
 from the third row of the table.
@@ -299,7 +318,7 @@ reading in a data structure as well.
 
 =cut
 
-our $rfits_options = new PDL::Options( { bscale=>1, data=>1, hdrcpy=>0, expand=>1 } );
+our $rfits_options = new PDL::Options( { bscale=>1, data=>1, hdrcpy=>0, expand=>1, afh=>1 } );
 
 sub PDL::rfitshdr {
   my $class = shift;
@@ -313,7 +332,7 @@ sub PDL::rfits {
   
   my $class = shift;
   
-  barf 'Usage: $a = rfits($file)  -or-   $a = PDL->rfits($file)' if (@_ < 1 || @_ > 2);
+  barf 'Usage: $x = rfits($file)  -or-   $x = PDL->rfits($file)' if (@_ < 1 || @_ > 2);
   
   my $file = shift; 
   
@@ -328,7 +347,9 @@ sub PDL::rfits {
   # indicator which cancelled the check for empty primary data array at the end.
   my $explicit_extension = ($file =~ m/\[\d+\]$/ ? 1 : 0);
   $extnum = ( ($file =~ s/\[(\d+)\]$//) ? $1 : 0 );
-  
+
+  $file =~ s/^(~)/glob($1)/e; #tilde expansion.
+
   $file = "gunzip -c $file |" if $file =~ /\.gz$/;    # Handle compression
   $file = "uncompress -c $file |" if $file =~ /\.Z$/;
   
@@ -415,7 +436,7 @@ sub PDL::rfits {
    # does not exist.
    # 
 
-   if($PDL::Astro_FITS_Header) { 
+   if($PDL::Astro_FITS_Header and $opt->{afh}) { 
 
      ## Astro::FITS::Header parsing.  Snarf lines to the END card,
      ## and pass them to Astro::FITS::Header.
@@ -435,7 +456,7 @@ sub PDL::rfits {
    } else {
      
      ## Legacy (straight header-to-hash-ref) parsing.  
-     ## Deprecated but preserved.
+     ## Cheesy but fast.
      
      hdr_legacy: { do {
        no strict 'refs';
@@ -532,7 +553,18 @@ sub PDL::rfits {
    
  $fh->close;
   
- return @extensions if(wantarray);
+ if(wantarray) { 
+     ## By default, ditch primary HDU placeholder 
+     if( ref($extensions[0]) eq 'HASH'  and 
+	 $extensions[0]->{SIMPLE} and
+	 exists($extensions[0]->{NAXIS}) and
+	 $extensions[0]->{NAXIS} == 0
+	 ) {
+	 shift @extensions;
+     }
+     # Return all the extensions 
+     return @extensions;
+ } 
  return $pdl;
 }
 
@@ -567,6 +599,7 @@ our $type_table = {
     8=>$PDL_B,
     16=>$PDL_S,
     32=>$PDL_L,
+    64=>$PDL_LL,
     -32=>$PDL_F,
     -64=>$PDL_D
 };
@@ -575,6 +608,7 @@ our $type_table_2 = {
     8=>byte,
     16=>short,
     32=>long,
+    64=>longlong,
     -32=>float,
     -64=>double
 };
@@ -624,9 +658,8 @@ sub _rfits_image($$$$) {
   if (!isbigendian() ) {
     # Need to byte swap on little endian machines
     bswap2($pdl) if $pdl->get_datatype == $PDL_S;
-    bswap4($pdl) if $pdl->get_datatype == $PDL_L || 
-      $pdl->get_datatype == $PDL_F;
-    bswap8($pdl) if $pdl->get_datatype == $PDL_D;
+    bswap4($pdl) if $pdl->get_datatype == $PDL_L || $pdl->get_datatype == $PDL_F;
+    bswap8($pdl) if $pdl->get_datatype == $PDL_D || $pdl->get_datatype==$PDL_LL;
   }
   
   if(exists $opt->{bscale}) {
@@ -648,30 +681,27 @@ sub treat_bscale($$){
 
     print "treating bscale...\n" if($PDL::debug);
 
-    if ( $PDL::Bad::Status ) {
-      # do we have bad values? - needs to be done before BSCALE/BZERO
-      # (at least for integers)
-      #
-      if ( $$foo{BITPIX} > 0 and exists $$foo{BLANK} ) {
-	# integer, so bad value == BLANK keyword
-	my $blank = $foo->{BLANK};
-	# do we have to do any conversion?
-	if ( $blank == $pdl->badvalue() ) {
-	  $pdl->badflag(1);
-	} else {
-	  # we change all BLANK values to the current bad value
-	  # (would not be needed with a per-piddle bad value)
-	  $pdl->inplace->setvaltobad( $blank );
-	}
-      } elsif ( $foo->{BITPIX} < 0 ) {
-	# bad values are stored as NaN's in FITS
-	# let setnanbad decide if we need to change anything
-	$pdl->inplace->setnantobad();
+    # do we have bad values? - needs to be done before BSCALE/BZERO
+    # (at least for integers)
+    if ( $$foo{BITPIX} > 0 and exists $$foo{BLANK} ) {
+      # integer, so bad value == BLANK keyword
+      my $blank = $foo->{BLANK};
+      # do we have to do any conversion?
+      if ( $blank == $pdl->badvalue() ) {
+        $pdl->badflag(1);
+      } else {
+        # we change all BLANK values to the current bad value
+        # (would not be needed with a per-piddle bad value)
+        $pdl->inplace->setvaltobad( $blank );
       }
-      print "FITS file may contain bad values.\n"
-	if $pdl->badflag() and $PDL::verbose;
-    } # if: PDL::Bad::Status
-    
+    } elsif ( $foo->{BITPIX} < 0 ) {
+      # bad values are stored as NaN's in FITS
+      # let setnanbad decide if we need to change anything
+      $pdl->inplace->setnantobad();
+    }
+    print "FITS file may contain bad values.\n"
+      if $pdl->badflag() and $PDL::verbose;
+
     my ($bscale, $bzero);
     $bscale = $$foo{"BSCALE"}; $bzero = $$foo{"BZERO"};
     print "BSCALE = $bscale &&  BZERO = $bzero\n" if $PDL::verbose;
@@ -805,6 +835,7 @@ $PDL::IO::FITS_bintable_handlers = {
   ,'L' => [ byte,    1,     1,     1  ] # logical - treat as byte
   ,'I' => [ short,   2,     2,     2  ] # short (no unsigned shorts?)
   ,'J' => [ long,    4,     4,     4  ] # long
+  ,'K' => [ longlong,8,     8,     8  ] # longlong
   ,'E' => [ float,   4,     4,     4  ] # single-precision
   ,'D' => [ double,  8,     8,     8  ] # double-precision
   ,'C' => [ sub { _nucomplx(float,  eval '@_') }, sub { _rdcomplx(float,  eval '@_') },
@@ -1199,9 +1230,9 @@ sub _rfits_bintable ($$$$) {
 	my %foo = ("TZERO$i"=>"BZERO", 
 		   "TSCAL$i"=>"BSCALE", 
 		   "TUNIT$i"=>"BUNIT");
-	for my $a(keys %foo) {
-	  $tmpcol->{data}->hdr->{$foo{$a}} = $hdr->{$a}
-	  if( defined($hdr->{$a}) );
+	for my $x(keys %foo) {
+	  $tmpcol->{data}->hdr->{$foo{$x}} = $hdr->{$x}
+	  if( defined($hdr->{$x}) );
 	}
       } # End of bscale checking...
 
@@ -1211,8 +1242,8 @@ sub _rfits_bintable ($$$$) {
 
       if(exists($hdr->{"TDIM$i"})) {
 	  if($hdr->{"TDIM$i"} =~ m/\((\s*\d+(\s*\,\s*\d+)*\s*)\)/) {
-	      my $a = $1;
-	      @tdims = map { $_+0 } split(/\,/,$a);
+	      my $x = $1;
+	      @tdims = map { $_+0 } split(/\,/,$x);
 	      my $tdims = pdl(@tdims);
 	      my $tds = $tdims->prodover;
 	      if($tds > $tmpcol->{data}->dim(0)) {
@@ -1487,9 +1518,16 @@ sub _rfits_unpack_zimage($$$) {
 	}
     }
 
+    # Copy the ZNAXIS* keywords to NAXIS*
+    foreach (grep /^NAXIS/,keys %$hdr){
+	if (exists($hdr->{'Z'.$_}) && defined($hdr->{'Z'.$_})){
+	    $hdr->{$_} = $hdr->{'Z'.$_};
+	}
+    }
+
     # Clean up the ZFOO extensions and table cruft
-    for my $k(keys %{$pdl->hdr}) {
-	delete $pdl->hdr->{$k} if(
+    for my $k(keys %{$hdr}) {
+	delete $hdr->{$k} if(
 	    $k=~ m/^Z/ ||
 	    $k eq "TFIELDS" ||
 	    $k =~ m/^TTYPE/ ||
@@ -1497,7 +1535,7 @@ sub _rfits_unpack_zimage($$$) {
 	    );
     }
 
-    if(exists $hdr->{BSCALE}) {
+    if(exists $hdr->{BSCALE} || exists $hdr->{BLANK}) {
 	$pdl = treat_bscale($pdl, $hdr);
     }
     $pdl->sethdr($hdr);
@@ -1528,6 +1566,11 @@ Suffix magic:
   # Automatically compress through pipe to compress 
   wfits $pdl, 'filename.fits.Z';  
 
+Tilde expansion:
+
+  #expand leading ~ to home directory (using glob())
+  wfits $pdl, '~/filename.fits';
+
 =over 3
 
 =item * Ordinary (PDL) data handling: 
@@ -1553,7 +1596,7 @@ C<wfits> will remove any extraneous C<NAXISn> keywords (per the FITS
 standard), and also remove the other keywords associated with that
 axis: C<CTYPEn>, C<CRPIXn>, C<CRVALn>, C<CDELTn>, and C<CROTAn>.  This
 may cause confusion if the slice is NOT out of the last dimension:
-C<wfits($a(:,(0),:),'file.fits');> and you would be best off adjusting
+C<wfits($x(:,(0),:),'file.fits');> and you would be best off adjusting
 the header yourself before calling C<wfits>.
 
 You can tile-compress images according to the CFITSIO extension to the 
@@ -1563,6 +1606,8 @@ FITS standard, by adding an option hash to the arguments:
 
 =item compress 
 
+CURRENTLY UNIMPLEMENTED.  Below describes the envisioned usage.
+
 This can be either unity, in which case Rice compression is used,
 or a (case-insensitive) string matching the CFITSIO compression 
 type names.  Currently supported compression algorithms are:
@@ -1633,8 +1678,8 @@ as a multi-string column in FITS tables, so any nonscalar values in
 the list are stringified before being written.  For example, if you
 pass in a perl list of 7 PDLs, each PDL will be stringified before
 being written, just as if you printed it to the screen.  This is
-probably not what you want -- you should use L<glue|glue> to connect 
-the separate PDLs into a single one.  (e.g. C<$a-E<gt>glue(1,$b,$c)-E<gt>mv(1,0)>)
+probably not what you want -- you should use L</glue> to connect 
+the separate PDLs into a single one.  (e.g. C<$x-E<gt>glue(1,$y,$c)-E<gt>mv(1,0)>)
 
 The column names are case-insensitive, but by convention the keys of
 C<$hash> should normally be ALL CAPS, containing only digits, capital
@@ -1682,9 +1727,9 @@ other fields are passed into the final FITS header verbatim.
 
 As an example, the following
 
-  $a = long(1,2,4);
-  $b = double(1,2,4);
-  wfits { 'COLA'=>$a, 'COLB'=>$b }, "table1.fits";
+  $x = long(1,2,4);
+  $y = double(1,2,4);
+  wfits { 'COLA'=>$x, 'COLB'=>$y }, "table1.fits";
 
 will create a binary FITS table called F<table1.fits> which
 contains two columns called C<COLA> and C<COLB>. The order
@@ -1692,7 +1737,7 @@ of the columns is controlled by setting the C<TTYPEn>
 keywords in the header array, so 
 
   $h = { 'TTYPE1'=>'Y', 'TTYPE2'=>'X' };
-  wfits { 'X'=>$a, 'Y'=>$b, hdr=>$h }, "table2.fits";
+  wfits { 'X'=>$x, 'Y'=>$y, hdr=>$h }, "table2.fits";
 
 creates F<table2.fits> where the first column is
 called C<Y> and the second column is C<X>.
@@ -1814,21 +1859,23 @@ sub wheader ($$) {
 #
 sub PDL::wfits {
   barf 'Usage: wfits($pdl,$file,[$BITPIX],[{options}])' if $#_<1 || $#_>3;
-  my ($pdl,$file,$a,$b) = @_;
+  my ($pdl,$file,$x,$y) = @_;
   my ($opt, $BITPIX);
 
   local $\ = undef;  # fix sf.net bug #3394327 
 
-  if(ref $a eq 'HASH') {
-      $a = $opt;
-      $BITPIX = $b;
-  } elsif(ref $b eq 'HASH') {
-      $BITPIX = $a;
-      $opt = $b;
+  if(ref $x eq 'HASH') {
+      $opt = $x;
+      $BITPIX = $y;
+  } elsif(ref $y eq 'HASH' || ! defined $y) {
+      $BITPIX = $x;
+      $opt = $y;
   }
 
   my ($k, $buff, $off, $ndims, $sz);
-  
+
+  $file =~ s/^(~)/glob($1)/e; # tilde expansion
+
   local $SIG{PIPE};
 
   if ($file =~ /\.gz$/) {            # Handle suffix-style compression
@@ -1845,344 +1892,380 @@ sub PDL::wfits {
   
   #### Figure output type
 
-  unless( UNIVERSAL::isa($pdl,'PDL') ) {
-      my $ref = ref($pdl) || "";
-      if($ref eq 'HASH') {
-	  my $fh = IO::File->new( $file )
-	      or barf "Could not open $file\n";
-	  _wfits_nullhdu($fh);
-	  # default to binary table if none specified
-	  my $table_type = exists $pdl->{tbl} ?
-	      ($pdl->{tbl} =~ m/^a/i ? 'ascii' : 'binary') :
-	      "binary";
-	  _wfits_table($fh,$pdl,$table_type);
-	  $fh->close;
-	return;
-      } else {
-	  barf('wfits: multiple extensions not yet supported\n')
-      }
+  my @outputs = ();
+  my $issue_nullhdu;
+
+  if( UNIVERSAL::isa($pdl,'PDL') ) {
+      $issue_nullhdu = 0;
+      push(@outputs, $pdl);
+  } elsif( ref($pdl) eq 'HASH' ) {
+      $issue_nullhdu = 1;
+      push(@outputs, $pdl);
+  } elsif( ref($pdl) eq 'ARRAY' ) {
+      $issue_nullhdu = 1;
+      @outputs = @$pdl;
+  } elsif( length(ref($pdl))==0 ) {
+      barf "wfits: needs a HASH or PDL argument to write out\n";
+  } else {
+      barf "wfits: unknown ref type ".ref($pdl)."\n";
   }
 
-  ### Regular image writing.
-  
-  $BITPIX = "" unless defined $BITPIX;
-  if ($BITPIX eq "") {
-    $BITPIX =   8 if $pdl->get_datatype == $PDL_B;
-    $BITPIX =  16 if $pdl->get_datatype == $PDL_S || $pdl->get_datatype == $PDL_US;
-    $BITPIX =  32 if $pdl->get_datatype == $PDL_L;
-    $BITPIX = -32 if $pdl->get_datatype == $PDL_F;
-    $BITPIX = -64 if $pdl->get_datatype == $PDL_D;
-  }
-  my $convert = sub { return $_[0] }; # Default - do nothing
-  $convert = sub {byte($_[0])}   if $BITPIX ==   8;
-  $convert = sub {short($_[0])}  if $BITPIX ==  16;
-  $convert = sub {long($_[0])}   if $BITPIX ==  32;
-  $convert = sub {float($_[0])}  if $BITPIX == -32;
-  $convert = sub {double($_[0])} if $BITPIX == -64;
-  
-  # Automatically figure output scaling
-  
-  my $bzero = 0; my $bscale = 1;
-  if ($BITPIX>0) {
-    my $min = $pdl->min;
-    my $max = $pdl->max;
-    my ($dmin,$dmax) = (0, 2**8-1)     if $BITPIX == 8;
-    ($dmin,$dmax) = (-2**15, 2**15-1)  if $BITPIX == 16;
-    ($dmin,$dmax) = (-2**31, 2**31-1)  if $BITPIX == 32;
-    
-    if ($min<$dmin || $max>$dmax) {
-      $bzero = $min - $dmin;
-      $max -= $bzero;
-      $bscale = $max/$dmax if $max>$dmax;
-    }
-    print "BSCALE = $bscale &&  BZERO = $bzero\n" if $PDL::verbose;
-  }
-
-  # Check for tile-compression format for the image, and handle it.
-  # We add the image-compression format tags and reprocess the whole
-  # shebang as a binary table.
-  if($opt->{compress}) {
-      croak "Placeholder -- tile compression not yet supported\n";
-  }
-  
-  
   ## Open file & prepare to write binary info
   my $fh = IO::File->new( $file )
       or barf "Unable to create FITS file $file\n";
   binmode $fh;
-  
-  ##############################
-  ## Check header and prepare to write it out
-  
-  my($h) = $pdl->gethdr();
 
-  # Extra logic: if we got handed a vanilla hash that that is *not* an Astro::FITS::Header, but 
-  # looks like it's a FITS header encoded in a hash, then attempt to process it with 
-  # Astro::FITS::Header before writing it out -- this helps with cleanup of tags.
-  if($PDL::Astro_FITS_Header and 
-     defined($h) and
-     ref($h) eq 'HASH' and
-     !defined( tied %$h )
-      ) {
+  if($issue_nullhdu) {
+      _wfits_nullhdu($fh);
+  }
 
-      my $all_valid_fits = 1;
-      for my $k(keys %$h) {
-	  if(length($k) > 8 or
-	     $k !~ m/^[A-Z_][A-Z\d\_]*$/i
+  for $pdl(@outputs) {
+
+      if(ref($pdl) eq 'HASH') {
+	  my $table_type = ( exists($pdl->{tbl}) ? 
+			     ($pdl->{tbl} =~ m/^a/i ? 'ascii' : 'binary') : 
+			     "binary" 
+	      );
+	  _wfits_table($fh,$pdl,$table_type);
+      } elsif( UNIVERSAL::isa($pdl,'PDL') ) {
+
+	  ### Regular image writing.
+	  
+	  $BITPIX = "" unless defined $BITPIX;
+	  if ($BITPIX eq "") {
+	      $BITPIX =   8 if $pdl->get_datatype == $PDL_B;
+	      $BITPIX =  16 if $pdl->get_datatype == $PDL_S || $pdl->get_datatype == $PDL_US;
+	      $BITPIX =  32 if $pdl->get_datatype == $PDL_L;
+	      $BITPIX =  64 if $pdl->get_datatype == $PDL_LL;
+	      $BITPIX = -32 if $pdl->get_datatype == $PDL_F;
+	      $BITPIX = -64 if $pdl->get_datatype == $PDL_D;
+	      $BITPIX = 8 * PDL::Core::howbig($PDL_IND) if($pdl->get_datatype==$PDL_IND);
+	  }
+	  if ($BITPIX eq "") {
+	      $BITPIX = -64;
+	      warn "wfits: PDL has an unsupported datatype -- defaulting to 64-bit float.\n";
+	  }
+
+	  my $convert = sub { return $_[0] }; # Default - do nothing
+	  $convert = sub {byte($_[0])}     if $BITPIX ==   8;
+	  $convert = sub {short($_[0])}    if $BITPIX ==  16;
+	  $convert = sub {long($_[0])}     if $BITPIX ==  32;
+	  $convert = sub {longlong($_[0])} if $BITPIX == 64;
+	  $convert = sub {float($_[0])}    if $BITPIX == -32;
+	  $convert = sub {double($_[0])}   if $BITPIX == -64;
+	  
+	  # Automatically figure output scaling
+	  
+	  my $bzero = 0; my $bscale = 1;
+	  if ($BITPIX>0) {
+	      my $min = $pdl->min;
+	      my $max = $pdl->max;
+	      my ($dmin,$dmax) = (0, 2**8-1)     if $BITPIX == 8;
+	      ($dmin,$dmax) = (-2**15, 2**15-1)  if $BITPIX == 16;
+	      ($dmin,$dmax) = (-2**31, 2**31-1)  if $BITPIX == 32;
+	      ($dmin,$dmax) = (-(pdl(longlong,1)<<63), (pdl(longlong,1)<<63)-1) if $BITPIX==64;
+
+	      if ($min<$dmin || $max>$dmax) {
+		  $bzero = $min - $dmin;
+		  $max -= $bzero;
+		  $bscale = $max/$dmax if $max>$dmax;
+	      }
+	      print "BSCALE = $bscale &&  BZERO = $bzero\n" if $PDL::verbose;
+	  }
+	  
+	  # Check for tile-compression format for the image, and handle it.
+	  # We add the image-compression format tags and reprocess the whole
+	  # shebang as a binary table.
+	  if($opt->{compress}) {
+	      croak "Placeholder -- tile compression not yet supported\n";
+	  }
+	  
+	  
+	  ##############################
+	  ## Check header and prepare to write it out
+	  
+	  my($h) = $pdl->gethdr();
+	  
+	  # Extra logic: if we got handed a vanilla hash that that is *not* an Astro::FITS::Header, but 
+	  # looks like it's a FITS header encoded in a hash, then attempt to process it with 
+	  # Astro::FITS::Header before writing it out -- this helps with cleanup of tags.
+	  if($PDL::Astro_FITS_Header and 
+	     defined($h) and
+	     ref($h) eq 'HASH' and
+	     !defined( tied %$h )
 	      ) {
-	      $all_valid_fits = 0;
-	      last;
+	      
+	      my $all_valid_fits = 1;
+	      for my $k(keys %$h) {
+		  if(length($k) > 8 or
+		     $k !~ m/^[A-Z_][A-Z\d\_]*$/i
+		      ) {
+		      $all_valid_fits = 0;
+		      last;
+		  }
+	      }
+	      
+	      if($all_valid_fits) {
+		  # All the keys look like valid FITS header keywords -- so 
+		  # create a tied FITS header object and use that instead.
+		  my $afh = new Astro::FITS::Header( );
+		  my %hh;
+		  tie %hh, "Astro::FITS::Header", $afh;
+		  for (keys %$h) {
+		      $hh{$_} = $h->{$_};
+		  }
+		  $h = \%hh;
+	      }
 	  }
-      }
-
-      if($all_valid_fits) {
-	  # All the keys look like valid FITS header keywords -- so 
-	  # create a tied FITS header object and use that instead.
-	  my $afh = new Astro::FITS::Header( );
-	  my %hh;
-	  tie %hh, "Astro::FITS::Header", $afh;
-	  for (keys %$h) {
-	      $hh{$_} = $h->{$_};
-	  }
-	  $h = \%hh;
-      }
-  }
-
-  # Now decide whether to emit a hash or an AFH object
-  if(defined($h) && 
-     ( (defined (tied %$h)) && 
-       (UNIVERSAL::isa(tied %$h,"Astro::FITS::Header")))
-     ){
-    my $k;
-    
-    ##n############################
-    ## Tied-hash code -- I'm too lazy to incorporate this into KGB's
-    ## direct hash handler below, so I've more or less just copied and
-    ## pasted with some translation.  --CED
-    ##
-    my $hdr = tied %$h;
-    
-    #
-    # Put advertising comment in the SIMPLE field
-    #n
-    $h->{SIMPLE} = 'T';
-    my(@a) = $hdr->itembyname('SIMPLE');
-    $a[0]->comment('Created with PDL (http://pdl.perl.org)');
-    
-    # and register it as a LOGICAL rather than a string
-    $a[0]->type('LOGICAL');
-    
-    #
-    # Use tied interface to set all the keywords.  Note that this
-    # preserves existing per-line comments, only changing the values.
-    #
-    $h->{BITPIX} = $BITPIX;
-    $h->{NAXIS} = $pdl->getndims;
-    my $correction = 0;
-    for $k(1..$h->{NAXIS}) { 
-      $correction |= (exists $h->{"NAXIS$k"} and 
-                      $h->{"NAXIS$k"} != $pdl->dim($k-1)
+	  
+	  # Now decide whether to emit a hash or an AFH object
+	  if(defined($h) && 
+	     ( (defined (tied %$h)) && 
+	       (UNIVERSAL::isa(tied %$h,"Astro::FITS::Header")))
+	      ){
+	      my $k;
+	      
+	      ##n############################
+	      ## Tied-hash code -- I'm too lazy to incorporate this into KGB's
+	      ## direct hash handler below, so I've more or less just copied and
+	      ## pasted with some translation.  --CED
+	      ##
+	      my $hdr = tied %$h;
+	      
+	      #
+	      # Put advertising comment in the SIMPLE field
+	      #n
+	      if($issue_nullhdu) {
+		  $h->{XTENSION} = "IMAGE";
+	      } else {
+		  $h->{SIMPLE} = 'T';
+		  my(@a) = $hdr->itembyname('SIMPLE');
+		  $a[0]->comment('Created with PDL (http://pdl.perl.org)');
+		  # and register it as a LOGICAL rather than a string
+		  $a[0]->type('LOGICAL');
+	      }
+	      
+	      #
+	      # Use tied interface to set all the keywords.  Note that this
+	      # preserves existing per-line comments, only changing the values.
+	      #
+	      $h->{BITPIX} = $BITPIX;
+	      $h->{NAXIS} = $pdl->getndims;
+	      my $correction = 0;
+	      for $k(1..$h->{NAXIS}) { 
+		  $correction |= (exists $h->{"NAXIS$k"} and 
+				  $h->{"NAXIS$k"} != $pdl->dim($k-1)
                       );
-      $h->{"NAXIS$k"} = $pdl->getdim($k-1); 
-    }
-    carp("Warning: wfits corrected dimensions of FITS header") 
-      if($correction);
-    
-    $h->{BUNIT} = "Data Value" unless exists $h->{BUNIT};
-    $h->{BSCALE} = $bscale if($bscale != 1);
-    $h->{BZERO}  = $bzero  if($bzero  != 0);
-    
-    if ( $pdl->badflag() ) {
-      if ( $BITPIX > 0 ) { my $a = &$convert(pdl(0.0));
-			   $h->{BLANK} = $a->badvalue(); }
-      else               { delete $h->{BLANK}; }
-    }
-    
-    # Use object interface to sort the lines. This is complicated by
-    # the need for an arbitrary number of NAXIS<n> lines in the middle
-    # of the sorting.  Keywords with a trailing '1' in the sorted-order
-    # list get looped over.
-    my($kk) = 0; 
-    my(@removed_naxis) = ();
-    for $k(0..$#PDL::IO::FITS::wfits_keyword_order) {
-      my($kn) = 0;
-      
-      my @index;
-      do {            # Loop over numericised keywords (e.g. NAXIS1)
-        
-        my $kw = $PDL::IO::FITS::wfits_keyword_order[$k]; # $kw get keyword
-        $kw .= (++$kn) if( $kw =~ s/\d$//);               # NAXIS1 -> NAXIS<n>
-        @index = $hdr->index($kw);
-        
-        if(defined $index[0]) {
-	    if($kn <= $pdl->getndims){
-		$hdr->insert($kk, $hdr->remove($index[0])) 
-		    unless ($index[0] == $kk) ;
-		$kk++;
-	    } 
-	    else{ #remove e.g. NAXIS3 from hdr if NAXIS==2
-		$hdr->removebyname($kw);
-		push(@removed_naxis,$kw);
-	    }
-	}
-      } while((defined $index[0]) && $kn);
-    }
+		  $h->{"NAXIS$k"} = $pdl->getdim($k-1); 
+	      }
+	      carp("Warning: wfits corrected dimensions of FITS header") 
+		  if($correction);
+	      
+	      $h->{BUNIT} = "Data Value" unless exists $h->{BUNIT};
+	      $h->{BSCALE} = $bscale if($bscale != 1);
+	      $h->{BZERO}  = $bzero  if($bzero  != 0);
+	      
+	      if ( $pdl->badflag() ) {
+		  if ( $BITPIX > 0 ) { my $x = &$convert(pdl(0.0));
+				       $h->{BLANK} = $x->badvalue(); }
+		  else               { delete $h->{BLANK}; }
+	      }
+	      
+	      # Use object interface to sort the lines. This is complicated by
+	      # the need for an arbitrary number of NAXIS<n> lines in the middle
+	      # of the sorting.  Keywords with a trailing '1' in the sorted-order
+	      # list get looped over.
+	      my($kk) = 0; 
+	      my(@removed_naxis) = ();
+	      for $k(0..$#PDL::IO::FITS::wfits_keyword_order) {
+		  my($kn) = 0;
+		  
+		  my @index;
+		  do {            # Loop over numericised keywords (e.g. NAXIS1)
+		      
+		      my $kw = $PDL::IO::FITS::wfits_keyword_order[$k]; # $kw get keyword
+		      $kw .= (++$kn) if( $kw =~ s/\d$//);               # NAXIS1 -> NAXIS<n>
+		      @index = $hdr->index($kw);
+		      
+		      if(defined $index[0]) {
+			  if($kn <= $pdl->getndims){
+			      $hdr->insert($kk, $hdr->remove($index[0])) 
+				  unless ($index[0] == $kk) ;
+			      $kk++;
+			  } 
+			  else{ #remove e.g. NAXIS3 from hdr if NAXIS==2
+			      $hdr->removebyname($kw);
+			      push(@removed_naxis,$kw);
+			  }
+		      }
+		  } while((defined $index[0]) && $kn);
+	      }
+	      
+	      foreach my $naxis(@removed_naxis){
+		  $naxis =~ m/(\d)$/;
+		  my $n = $1;
+		  foreach my $kw(@PDL::IO::FITS::wfits_numbered_keywords){
+		      $hdr->removebyname($kw . $n);
+		  }
+	      }
+	      #
+	      # Delete the END card if necessary (for later addition at the end)
+	      #
+	      $hdr->removebyname('END');
+	      
+	      #
+	      # Make sure that the HISTORY lines all come at the end
+	      # 
+	      my @hindex = $hdr->index('HISTORY');
+	      for $k(0..$#hindex) {
+		  $hdr->insert(-1-$k, $hdr->remove($hindex[-1-$k]));
+	      }
+	      
+	      #
+	      # Make sure the last card is an END
+	      #
+	      $hdr->insert(scalar($hdr->cards),
+			   Astro::FITS::Header::Item->new(Keyword=>'END'));
+	      
+	      #
+	      # Write out all the cards, and note how many bytes for later padding.
+	      #
+	      my $s = join("",$hdr->cards);
+	      
+	      $fh->print( $s );
+	      $nbytes = length $s;
+	  } else {
+	      ##
+	      ## Legacy emitter (note different advertisement in the SIMPLE
+	      ## comment, for debugging!)
+	      ##
 
-    foreach my $naxis(@removed_naxis){
-	$naxis =~ m/(\d)$/;
-	my $n = $1;
-	foreach my $kw(@PDL::IO::FITS::wfits_numbered_keywords){
-	    $hdr->removebyname($kw . $n);
-	}
-    }
-    #
-    # Delete the END card if necessary (for later addition at the end)
-    #
-    $hdr->removebyname('END');
-    
-    #
-    # Make sure that the HISTORY lines all come at the end
-    # 
-    my @hindex = $hdr->index('HISTORY');
-    for $k(0..$#hindex) {
-      $hdr->insert(-1-$k, $hdr->remove($hindex[-1-$k]));
-    }
-    
-    #
-    # Make sure the last card is an END
-    #
-    $hdr->insert(scalar($hdr->cards),
-                 Astro::FITS::Header::Item->new(Keyword=>'END'));
-    
-    #
-    # Write out all the cards, and note how many bytes for later padding.
-    #
-    my $s = join("",$hdr->cards);
-    
-    $fh->print( $s );
-    $nbytes = length $s;
-  } else {
-    ##
-    ## Legacy emitter (note different advertisement in the SIMPLE
-    ## comment, for debugging!)
-    ##
-    
-    $fh->printf( "%-80s", "SIMPLE  =                    T / PDL::IO::FITS::wfits (http://pdl.perl.org)" );
-    
-    $nbytes = 80; # Number of bytes written so far
-    
-    # Write FITS header
-    
-    %hdr = ();
-    if (defined($h)) {
-      for (keys %$h) { $hdr{uc $_} = $$h{$_} } # Copy (ensuring keynames are uppercase)
-    }
-
-    delete $hdr{SIMPLE}; delete $hdr{'END'};
-    
-    $hdr{BITPIX} =  $BITPIX;
-    $hdr{BUNIT} = "Data Value" unless exists $hdr{BUNIT};
-    wheader($fh, 'BITPIX');
-    
-    $ndims = $pdl->getndims; # Dimensions of data array
-    $hdr{NAXIS}  = $ndims;
-    wheader($fh, 'NAXIS');
-    for $k (1..$ndims) { $hdr{"NAXIS$k"} = $pdl->getdim($k-1) }
-    for $k (1..$ndims) { wheader($fh,"NAXIS$k") }
-    
-    if ($bscale != 1 || $bzero  != 0) {
-      $hdr{BSCALE} =  $bscale;
-      $hdr{BZERO}  =  $bzero;
-      wheader($fh,'BSCALE');
-      wheader($fh,'BZERO');
-    }
-    wheader($fh,'BUNIT');
-    
-    # IF badflag is set
-    #   and BITPIX > 0 - ensure the header contains the BLANK keyword
-    #                    (make sure it's for the correct type)
-    #   otherwise      - make sure the BLANK keyword is removed
-    if ( $pdl->badflag() ) {
-      if ( $BITPIX > 0 ) { my $a = &$convert(pdl(0.0)); $hdr{BLANK} = $a->badvalue(); }
-      else               { delete $hdr{BLANK}; }
-    }
-    
-    for $k (sort fits_field_cmp keys %hdr) { 
-	wheader($fh,$k) unless $k =~ m/HISTORY/;
-    }
-    wheader($fh, 'HISTORY'); # Make sure that HISTORY entries come last.
-    $fh->printf( "%-80s", "END" );
-    $nbytes += 80;
-  }
-  
-  #
-  # Pad the header to a legal value and write the rest of the FITS file.
-  #
-  $nbytes %= 2880;
-  $fh->print( " "x(2880-$nbytes) )
-      if $nbytes != 0; # Fill up HDU
-  
-  # Decide how to byte swap - note does not quite work yet. Needs hack
-  # to IO.xs
-  
-  my $bswap = sub {};     # Null routine
-  if ( !isbigendian() ) { # Need to set a byte swap routine
-    $bswap = \&bswap2 if $BITPIX==16;
-    $bswap = \&bswap4 if $BITPIX==32 || $BITPIX==-32;
-    $bswap = \&bswap8 if $BITPIX==-64;
-  }
-  
-  # Write FITS data
-  
-  my $p1d = $pdl->copy->reshape($pdl->nelem); # Data as 1D stream;
-  
-  $off = 0;
-  $sz  = PDL::Core::howbig(&$convert($p1d->slice('0:0'))->get_datatype);
-  
-  $nbytes = $p1d->getdim(0) * $sz;
-  
-  # Transfer data in blocks (because might need to byte swap)
-  # Buffer is also type converted on the fly
-  
-  my $BUFFSZ = 360*2880; # = ~1Mb - must be multiple of 2880
-  my $tmp;
-  
-  if ( $pdl->badflag() and $BITPIX < 0 and $PDL::Bad::UseNaN == 0 ) {
-    # just print up a message - conversion is actually done in the loop
-    print "Converting PDL bad value to NaN\n" if $PDL::verbose;
-  }
-  
-  while ($nbytes - $off > $BUFFSZ) {
-    
-    # Data to be transferred
-    
-    $buff = &$convert( ($p1d->slice( ($off/$sz).":". (($off+$BUFFSZ)/$sz-1))
-                        -$bzero)/$bscale );
-    
-    # if there are bad values present, and output type is floating-point,
-    # convert the bad values to NaN's.  We can ignore integer types, since
-    # we have set the BLANK keyword
-    #
-    if ( $pdl->badflag() and $BITPIX < 0 and $PDL::Bad::UseNaN == 0 ) {
-      $buff->inplace->setbadtonan();
-    }
-    
-    &$bswap($buff);
-    $fh->print( ${$buff->get_dataref} );
-    $off += $BUFFSZ;
-}
-$buff = &$convert( ($p1d->slice($off/$sz.":-1") - $bzero)/$bscale );
-
-if ( $pdl->badflag() and $BITPIX < 0 and $PDL::Bad::UseNaN == 0 ) {
-  $buff->inplace->setbadtonan();
-}
-
-  &$bswap($buff);
-  $fh->print( ${$buff->get_dataref} );
-  # Fill HDU and close
-  # note that for the data space the fill character is \0 not " "
-  #
-  $fh->print( "\0"x(($BUFFSZ - $buff->getdim(0) * $sz)%2880) );
+	      if($issue_nullhdu) {
+		  $fh->printf( "%-80s", "XTENSION= 'IMAGE'" );
+	      } else {
+		  $fh->printf( "%-80s", "SIMPLE  =                    T / PDL::IO::FITS::wfits (http://pdl.perl.org)" );
+	      }
+	      
+	      $nbytes = 80; # Number of bytes written so far
+	      
+	      # Write FITS header
+	      
+	      %hdr = ();
+	      if (defined($h)) {
+		  for (keys %$h) { $hdr{uc $_} = $$h{$_} } # Copy (ensuring keynames are uppercase)
+	      }
+	      
+	      delete $hdr{SIMPLE}; delete $hdr{'END'};
+	      
+	      $hdr{BITPIX} =  $BITPIX;
+	      $hdr{BUNIT} = "Data Value" unless exists $hdr{BUNIT};
+	      wheader($fh, 'BITPIX');
+	      
+	      $ndims = $pdl->getndims; # Dimensions of data array
+	      $hdr{NAXIS}  = $ndims;
+	      wheader($fh, 'NAXIS');
+	      for $k (1..$ndims) { $hdr{"NAXIS$k"} = $pdl->getdim($k-1) }
+	      for $k (1..$ndims) { wheader($fh,"NAXIS$k") }
+	      
+	      if ($bscale != 1 || $bzero  != 0) {
+		  $hdr{BSCALE} =  $bscale;
+		  $hdr{BZERO}  =  $bzero;
+		  wheader($fh,'BSCALE');
+		  wheader($fh,'BZERO');
+	      }
+	      wheader($fh,'BUNIT');
+	      
+	      # IF badflag is set
+	      #   and BITPIX > 0 - ensure the header contains the BLANK keyword
+	      #                    (make sure it's for the correct type)
+	      #   otherwise      - make sure the BLANK keyword is removed
+	      if ( $pdl->badflag() ) {
+		  if ( $BITPIX > 0 ) { my $x = &$convert(pdl(0.0)); $hdr{BLANK} = $x->badvalue(); }
+		  else               { delete $hdr{BLANK}; }
+	      }
+	      
+	      for $k (sort fits_field_cmp keys %hdr) { 
+		  wheader($fh,$k) unless $k =~ m/HISTORY/;
+	      }
+	      wheader($fh, 'HISTORY'); # Make sure that HISTORY entries come last.
+	      $fh->printf( "%-80s", "END" );
+	      $nbytes += 80;
+	  }
+	  
+	  #
+	  # Pad the header to a legal value and write the rest of the FITS file.
+	  #
+	  $nbytes %= 2880;
+	  $fh->print( " "x(2880-$nbytes) )
+	      if $nbytes != 0; # Fill up HDU
+	  
+	  # Decide how to byte swap - note does not quite work yet. Needs hack
+	  # to IO.xs
+	  
+	  my $bswap = sub {};     # Null routine
+	  if ( !isbigendian() ) { # Need to set a byte swap routine
+	      $bswap = \&bswap2 if $BITPIX==16;
+	      $bswap = \&bswap4 if $BITPIX==32 || $BITPIX==-32;
+	      $bswap = \&bswap8 if $BITPIX==-64 || $BITPIX==64;
+	  }
+	  
+	  # Write FITS data
+	  
+	  my $p1d = $pdl->copy->reshape($pdl->nelem); # Data as 1D stream;
+	  
+	  $off = 0;
+	  $sz  = PDL::Core::howbig(&$convert($p1d->slice('0:0'))->get_datatype);
+	  
+	  $nbytes = $p1d->getdim(0) * $sz;
+	  
+	  # Transfer data in blocks (because might need to byte swap)
+	  # Buffer is also type converted on the fly
+	  
+	  my $BUFFSZ = 360*2880; # = ~1Mb - must be multiple of 2880
+	  my $tmp;
+	  
+	  if ( $pdl->badflag() and $BITPIX < 0 and $PDL::Bad::UseNaN == 0 ) {
+	      # just print up a message - conversion is actually done in the loop
+	      print "Converting PDL bad value to NaN\n" if $PDL::verbose;
+	  }
+	  
+	  while ($nbytes - $off > $BUFFSZ) {
+	      
+	      # Data to be transferred
+	      
+	      $buff = &$convert( ($p1d->slice( ($off/$sz).":". (($off+$BUFFSZ)/$sz-1))
+				  -$bzero)/$bscale );
+	      
+	      # if there are bad values present, and output type is floating-point,
+	      # convert the bad values to NaN's.  We can ignore integer types, since
+	      # we have set the BLANK keyword
+	      #
+	      if ( $pdl->badflag() and $BITPIX < 0 and $PDL::Bad::UseNaN == 0 ) {
+		  $buff->inplace->setbadtonan();
+	      }
+	      
+	      &$bswap($buff);
+	      $fh->print( ${$buff->get_dataref} );
+	      $off += $BUFFSZ;
+	  }
+	  $buff = &$convert( ($p1d->slice($off/$sz.":-1") - $bzero)/$bscale );
+	  
+	  if ( $pdl->badflag() and $BITPIX < 0 and $PDL::Bad::UseNaN == 0 ) {
+	      $buff->inplace->setbadtonan();
+	  }
+	  
+	  &$bswap($buff);
+	  $fh->print( ${$buff->get_dataref} );
+	  # Fill HDU and close
+	  # note that for the data space the fill character is \0 not " "
+	  #
+	  $fh->print( "\0"x(($BUFFSZ - $buff->getdim(0) * $sz)%2880) );
+      } # end of image writing block 
+      else { 
+	  # Not a PDL and not a hash ref
+	  barf("wfits: unknown data type - quitting");
+     }
+  } # end of output loop
   $fh->close();
   1;
 }
@@ -2365,7 +2448,7 @@ sub _prep_table {
 
   ### Allocate any table columns that are already in the header...
   local $_;
-  map { for my $a(1) { # [Shenanigans to make next work right]
+  map { for my $x(1) { # [Shenanigans to make next work right]
     next unless m/^TTYPE(\d*)$/;
 
     my $num = $1;
@@ -2619,12 +2702,12 @@ FOO
       my $row = "";
       for my $c(1..$cols) {
 	my $tmp;
-	my $a = $fieldvars[$c];
+	my $x = $fieldvars[$c];
        
 	if($internaltype[$c] eq 'P') {  # PDL handling
 	  $tmp = $converters[$c]
-	    ? &{$converters[$c]}($a->slice("$r")->flat->sever, $r, $c) 
-	      : $a->slice("$r")->flat->sever ;
+	    ? &{$converters[$c]}($x->slice("$r")->flat->sever, $r, $c)
+	      : $x->slice("$r")->flat->sever ;
 
 	  ## This would go faster if moved outside the loop but I'm too
 	  ## lazy to do it Right just now.  Perhaps after it actually works.
@@ -2639,9 +2722,9 @@ FOO
 	  my $t = $tmp->get_dataref;  
 	  $tmp = $$t;
 	} else {                                  # Only other case is ASCII just now...
-	  $tmp = ( ref $a eq 'ARRAY' ) ?          # Switch on array or string
-	    ( $#$a == 0 ? $a->[0] : $a->[$r] )    # Thread arrays as needed
-	      : $a;
+	  $tmp = ( ref $x eq 'ARRAY' ) ?          # Switch on array or string
+	    ( $#$x == 0 ? $x->[0] : $x->[$r] )    # Thread arrays as needed
+	      : $x;
 	 
 	  $tmp .= " " x ($field_len[$c] - length($tmp));
 	}
